'use strict';

var mandrill = require('mandrill-api/mandrill');
var errTree = require('err-tree');
var _ = require('lodash');
var defaultValue = require('./utils/templateHelper').defaultValue;

function Email(options) {
    this.options = _.defaults({}, options);
    if (!this.client && this.options.mandrillApiKey) {
        this.client = new mandrill.Mandrill(this.options.mandrillApiKey);
    }
}

Email.prototype.EmailError = errTree(function EmailError(email, error) {
    errTree.BasicError.apply(this, [JSON.stringify({email: email, error: error})]);
});
/**
 * Creates a new e-mail message to be sent using a template
 *
 * @param subject the string message on the subject of the e-mail
 * @param templateName the name of the template to be used for the content of the message
 * @param templateContent an array containing all variable replacements of the template
 * @param sender.name the name of the sender
 * @param sender.email the e-mail address of the sender
 * @param recipient.name the name of the recipient
 * @param recipient.email the e-mail address of the recipient
 * @constructor
 *
 * @see https://mandrillapp.com/api/docs/messages.nodejs.html#method=send-template
 */
Email.prototype.SendTemplate = function SendTemplate(templateName, templateContent, config) {
    this.template_content = templateContent || [];
    switch (templateName) {
        case 'default':
        defaultValue('URL_BASE', this.template_content, '');
        defaultValue('IMG_HEADER', this.template_content,
            'https://static.coorpacademy.com/emails/default/img-header-v1-760x334.png');
        defaultValue('call2action', this.template_content, '');
        defaultValue('TITLE', this.template_content, '');
        defaultValue('SUBTILE', this.template_content, '');
        defaultValue('custom', this.template_content, '');
        break;
    }
    this.async = true;
    this.message = {};
    this.template_name = templateName;
    this.message.subject = config.subject;
    this.message.to = [{
        type: 'to',
        name: config.recipient.name,
        email: config.recipient.email
    }];
    this.message.from_email = config.sender.email;
    this.message.from_name = config.sender.name;
    // use same for other merge template engine
    this.message.global_merge_vars = this.template_content;
};

Email.prototype._onSuccessCallback = function(result, callback) {
    var status = !result || !result.length ? null : result[0].status;
    if (status !== 'sent' && status !== 'queued') {
        return callback(
            {
                status: 'error',
                code: -1,
                name: 'MandrillResultError',
                message: 'Mandrill reports a bad status for e-mail sent: ' + status
            }
        );
    }
    return callback(null, result);
};

Email.prototype.send = function(sendTemplate, callback) {
    this.client.messages.sendTemplate(
        sendTemplate,
        function(result) {
            return this._onSuccessCallback(result, callback);
        }.bind(this),
        callback
    );
};

module.exports = Email;
