'use strict';

var sinon = require('sinon');
var expect = require('chai').expect;
var mandrill = require('mandrill-api/mandrill');
var Email = require('../../lib/email/email');

describe('email', function() {
    describe('send', function() {
        var messagesApi = {sendTemplate: function() { }};
        var messagesMock;
        var email;

        beforeEach(function() {
            messagesMock = sinon.mock(messagesApi);
            sinon.stub(mandrill, 'Mandrill').returns({messages: messagesMock.object});
            email = new Email({mandrillApiKey: 'mandrillApiKey'});
        });
        afterEach(function() {
            mandrill.Mandrill.restore();
            email.client = undefined;
        });

        it('should send an e-mail with mandrill using template', function() {
            var sendTemplate = 'the template to be sent';
            messagesMock.expects('sendTemplate').once().withArgs(sendTemplate);
            email.send(sendTemplate);
            messagesMock.verify();
        });
    });

    describe('callbacks', function() {
        var callback = function(error, result) {
            return {error: error, result: result};
        };
        var messagesMock = {
            sendTemplate: function() {}
        };

        var email;

        beforeEach(function() {
            sinon.spy(messagesMock, 'sendTemplate');
            sinon.stub(mandrill, 'Mandrill').returns({messages: messagesMock});
            email = new Email({mandrillApiKey: 'mandrillApiKey'});
        });
        afterEach(function() {
            mandrill.Mandrill.restore();
            messagesMock.sendTemplate.restore();
            email.client = undefined;
        });

        it('should handle success result with status \'sent\'', function() {
            email.send('template', callback);

            // retrieve onsuccess function passed to mandrill
            var onSuccess = messagesMock.sendTemplate.getCall(0).args[1];
            var result = [{status: 'sent'}];
            expect(onSuccess(result)).to.be.eql({
                result: result,
                error: null
            });
        });

        it('should handle success result with status \'queued\'', function() {
            email.send('template', callback);

            // retrieve onsuccess function passed to mandrill
            var onSuccess = messagesMock.sendTemplate.getCall(0).args[1];
            var result = [{status: 'queued'}];
            expect(onSuccess(result)).to.be.eql({
                result: result,
                error: null
            });
        });

        it('should handle success result with bad status as an error', function() {
            email.send('template', callback);

            // retrieve onsuccess function passed to mandrill
            var onSuccess = messagesMock.sendTemplate.getCall(0).args[1];
            var result = [{status: 'weird'}];
            expect(onSuccess(result)).to.be.eql({
                result: undefined,
                error: {
                    status: 'error',
                    code: -1,
                    name: 'MandrillResultError',
                    message: 'Mandrill reports a bad status for e-mail sent: weird'
                }
            });
        });

        it('should handle error', function() {
            email.send('template', callback);

            // retrieve onerror function passed to mandrill
            var onError = messagesMock.sendTemplate.getCall(0).args[2];
            var error = 'error';
            expect(onError(error, null)).to.be.eql({
                result: null,
                error: error
            });
        });

        it('should handle prepare email', function() {
            email.SendTemplate('default', [{
                name: 'TITLE',
                content: 'some hard-coded title'
            }], {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                }
            });
            expect(email.template_name).to.be.equal('default');
        });
        it('should handle accept empty values', function() {
            email.SendTemplate('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                }
            });
            expect(email.template_name).to.be.equal('default');
        });
        it('should handle no result from mandrill', function(done) {
            email._onSuccessCallback(null, function(err) {
                expect(err.status).to.be.equal('error');
                done();
            });
        });

        it('should have EmailError', function() {
            var emailError = new email.EmailError();
            expect(emailError).to.have.property('name', 'EmailError');
        });
    });
});
